"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReplicasAction = void 0;
const abstract_action_1 = require("./abstract.action");
const project_utils_1 = require("../lib/utils/project-utils");
const auth_guard_1 = require("../lib/utils/auth-guard");
const chalk = require("chalk");
const ui_1 = require("../lib/ui");
const api_service_1 = require("../lib/apiService/api.service");
class ReplicasAction extends abstract_action_1.AbstractAction {
    handle(inputs, options, extraFlags) {
        return __awaiter(this, void 0, void 0, function* () {
            const answers = [];
            yield this.initFunction(answers, inputs);
            const runReplicas = this.runReplicas(inputs);
            runReplicas.then(() => __awaiter(this, void 0, void 0, function* () {
                const replicas = project_utils_1.getInputsValue(inputs, 'replicas');
                //    await updatePackageJson(inputs, options || []);TODO FIX
                this.printMessage(`replicas:${replicas} `);
                process.exit(0);
            })).catch((error) => {
                console.log(error);
                process.exit(1);
            });
        });
    }
    initFunction(answers, inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            yield project_utils_1.functionsName(inputs, answers);
            const id = yield project_utils_1.functionsId(inputs, answers);
            const idNamespaceValue = yield project_utils_1.idNamespace(answers, answers);
            const clikeyValue = yield project_utils_1.clikey(inputs, answers);
            if (clikeyValue === undefined || clikeyValue == '') {
                yield auth_guard_1.authGuard(inputs);
                yield this.initFunction(answers, inputs);
            }
            if (project_utils_1.isEmpty(id) || project_utils_1.isEmpty(idNamespaceValue)) {
                this.printMessage(`Check if your function is well start please ${chalk.bold(`${ui_1.EMOJIS.COFFEE}`)}`);
                process.exit(1);
            }
            yield project_utils_1.selectReplicas(inputs);
        });
    }
    printMessage(message) {
        const emptyLine = project_utils_1.print();
        const green = project_utils_1.print('green');
        project_utils_1.print()(message);
        project_utils_1.print()(`${chalk.bold(`${ui_1.EMOJIS.WINE}  blog:`)} ${chalk.underline('https://microfunctions.io/blog')}`);
        emptyLine();
    }
    runReplicas(inputs) {
        const functions = this.bodyFunctions(inputs);
        return api_service_1.runReplicas(functions, inputs);
    }
    bodyFunctions(inputs) {
        return {
            "idfunctions": project_utils_1.getInputsValue(inputs, 'id'),
            "replicas": project_utils_1.getInputsValue(inputs, 'replicas'),
            "idNamespace": project_utils_1.getInputsValue(inputs, 'idNamespace')
        };
    }
}
exports.ReplicasAction = ReplicasAction;
