"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatusAction = void 0;
const ui_1 = require("../lib/ui");
const project_utils_1 = require("../lib/utils/project-utils");
const chalk = require("chalk");
const abstract_action_1 = require("./abstract.action");
const auth_guard_1 = require("../lib/utils/auth-guard");
const new_action_1 = require("./new.action");
const api_service_1 = require("../lib/apiService/api.service");
class StatusAction extends abstract_action_1.AbstractAction {
    handle(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            console.info(ui_1.MESSAGES.FUNCTIONS_STATUS);
            console.info();
            const answers = [];
            try {
                yield this.initFunction(answers, inputs);
                const runStart = this.runStatus(inputs);
                runStart.then((depoly) => __awaiter(this, void 0, void 0, function* () {
                    if (depoly.code) {
                        throw new Error(depoly.message);
                    }
                    this.printMessage(depoly);
                    process.exit(0);
                })).catch((error) => {
                    var _a;
                    const messageErr = ((_a = error.response) === null || _a === void 0 ? void 0 : _a.body) || error;
                    this.printErrorMessage(messageErr);
                    process.exit(1);
                });
            }
            catch (e) {
                if (e.name === 'TypeError' || e.name === 'SyntaxError') {
                    console.error('error =>', e);
                }
                else {
                    console.error(e.message);
                }
                project_utils_1.print()(`${chalk.bold(`${ui_1.EMOJIS.COFFEE}  issues:`)} ${chalk.underline('https://github.com/microfunctionsio/microfunctions-cli/issues/new')}`);
                process.exit(1);
            }
        });
    }
    runStatus(inputs) {
        const functions = this.bodyFunctions(inputs);
        return api_service_1.runStatus(functions, inputs);
    }
    bodyFunctions(inputs) {
        return {
            "idfunctions": project_utils_1.getInputsValue(inputs, 'id'),
            "idNamespace": project_utils_1.getInputsValue(inputs, 'idNamespace')
        };
    }
    printErrorMessage(message) {
        var _a;
        const emptyLine = project_utils_1.print();
        const red = project_utils_1.print('red');
        red(`${message} ${ui_1.EMOJIS.BOOM}`);
        switch ((_a = message.status) === null || _a === void 0 ? void 0 : _a.toString()) {
            case '403':
                red(`check your cliKey please  ${ui_1.EMOJIS.COFFEE}`);
                break;
        }
        project_utils_1.print()(`${chalk.bold(`${ui_1.EMOJIS.WINE}  blog:`)} ${chalk.underline('https://microfunctions.io/blog')}`);
        emptyLine();
    }
    printMessage(status) {
        const emptyLine = project_utils_1.print();
        const green = project_utils_1.print('green');
        const red = project_utils_1.print('red');
        if (status.length > 0) {
            console.log(status);
            const podStatus = status.find((podStatus) => podStatus.status === 'Running');
            const podStatusko = status.find((podStatus) => podStatus.status !== 'Running');
            if (podStatus) {
                green(`functions ${podStatus.status}`);
            }
            else {
                red(`functions ${podStatusko === null || podStatusko === void 0 ? void 0 : podStatusko.status}`);
            }
        }
        else {
            console.log(status);
            emptyLine(`please check if your function is well start`);
        }
        project_utils_1.print()(`${chalk.bold(`${ui_1.EMOJIS.WINE}  blog:`)} ${chalk.underline('https://microfunctions.io/blog')}`);
        emptyLine();
    }
    initFunction(answers, inputs) {
        return __awaiter(this, void 0, void 0, function* () {
            yield project_utils_1.isInRootDirectory(inputs);
            const id = yield project_utils_1.functionsId(inputs, answers);
            const idNamespaceValue = yield project_utils_1.idNamespace(answers, answers);
            if (project_utils_1.isEmpty(id) || project_utils_1.isEmpty(idNamespaceValue)) {
                project_utils_1.printCheckMessage(`Check if your function is well deployed please ${chalk.bold(`${ui_1.EMOJIS.COFFEE}`)}`);
                process.exit(1);
            }
            yield project_utils_1.functionsName(inputs, answers);
            yield project_utils_1.executedName(inputs, answers);
            const clikeyValue = yield project_utils_1.clikey(inputs, answers);
            if (clikeyValue === undefined || clikeyValue == '') {
                yield auth_guard_1.authGuard(inputs);
                yield this.initFunction(answers, inputs);
            }
            const namespaceValue = yield project_utils_1.namespace(inputs, answers);
            if (namespaceValue === undefined || namespaceValue == '') {
                yield new_action_1.askForMissingInformation(inputs);
                yield this.initFunction(answers, inputs);
            }
            yield project_utils_1.loadSourceCode(inputs, answers);
            yield project_utils_1.loadEnv(inputs, answers);
            yield project_utils_1.loadDependencies(inputs, answers);
            yield project_utils_1.replicas(inputs, answers);
            yield project_utils_1.memory(inputs, answers);
            yield project_utils_1.runtime(inputs, answers);
        });
    }
}
exports.StatusAction = StatusAction;
