"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandLoader = void 0;
const chalk = require("chalk");
const actions_1 = require("../actions");
const ui_1 = require("../lib/ui");
const new_command_1 = require("./new.command");
const deploy_command_1 = require("./deploy.command");
const deploy_action_1 = require("../actions/deploy.action");
const replicas_action_1 = require("../actions/replicas.action");
const replicas_command_1 = require("./replicas.command");
const start_command_1 = require("./start.command");
const stop_action_1 = require("../actions/stop.action");
const stop_command_1 = require("./stop.command");
const status_action_1 = require("../actions/status.action");
const status_command_1 = require("./status.command");
const logs_command_1 = require("./logs.command");
const logs_action_1 = require("../actions/logs.action");
class CommandLoader {
    static load(program) {
        new new_command_1.NewCommand(new actions_1.NewAction()).load(program);
        new deploy_command_1.DeployCommand(new deploy_action_1.DeployAction()).load(program);
        new replicas_command_1.ReplicasCommand(new replicas_action_1.ReplicasAction()).load(program);
        new start_command_1.StartCommand(new actions_1.StartAction()).load(program);
        new stop_command_1.StopCommand(new stop_action_1.StopAction()).load(program);
        new status_command_1.StatusCommand(new status_action_1.StatusAction()).load(program);
        new logs_command_1.LogsCommand(new logs_action_1.LogsAction()).load(program);
        this.handleInvalidCommand(program);
    }
    static handleInvalidCommand(program) {
        program.on('command:*', () => {
            console.error(`\n${ui_1.ERROR_PREFIX} Invalid command: ${chalk.red('%s')}`, program.args.join(' '));
            console.log(`See ${chalk.red('--help')} for a list of available commands.\n`);
            process.exit(1);
        });
    }
}
exports.CommandLoader = CommandLoader;
