"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runReplicas = exports.runStatus = exports.runLogs = exports.runStop = exports.runStart = exports.runDeploy = exports.getAllCluster = exports.getAllNamespaces = exports.ApiService = void 0;
const node_fetch_1 = require("node-fetch");
const project_utils_1 = require("../utils/project-utils");
const url_api_1 = require("../utils/url.api");
class ApiService {
    constructor(clikey) {
        this._method = 'POST';
        this._url = url_api_1.Endpoints.baseAPi;
        this._action = url_api_1.Action.DEPLOY;
        this._headers = {};
        this._clikey = clikey;
    }
    get clikey() {
        return this._clikey;
    }
    set clikey(clikey) {
        this._clikey = clikey;
    }
    action(action) {
        this.setUrl(action);
        return this;
    }
    setUrl(url) {
        this._url = this._url.concat(url);
        return this;
    }
    setBody(body) {
        this._body = body;
        return this;
    }
    setMethod(method) {
        this._method = method;
        return this;
    }
    setHeaders(headers) {
        for (const i in headers) {
            if (headers[i].hasOwnProperty('key')
                && headers[i].hasOwnProperty('value')) {
                this._headers[headers[i].key] = headers[i].value;
            }
        }
        return this;
    }
    handle() {
        this.setHeaders([
            {
                key: 'x-clikey-header',
                value: this._clikey
            },
            {
                key: 'Content-Type',
                value: 'application/json'
            },
        ]);
        return node_fetch_1.default(this._url, {
            headers: this._headers,
            method: this._method,
            body: JSON.stringify(this._body || {}),
        }).then(res => res.json());
    }
}
exports.ApiService = ApiService;
exports.getAllNamespaces = (inputs) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const cliKey = (_a = project_utils_1.getApplicationClikeynput(inputs)) === null || _a === void 0 ? void 0 : _a.value;
    if (cliKey != '') {
        const api = new ApiService(cliKey)
            .action(url_api_1.Action.NAMESPACES);
        return api.handle();
    }
    return [];
});
exports.getAllCluster = (inputs) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    const cliKey = (_b = project_utils_1.getApplicationClikeynput(inputs)) === null || _b === void 0 ? void 0 : _b.value;
    if (cliKey != '') {
        const api = new ApiService(cliKey)
            .action(url_api_1.Action.CLUSTERS);
        return api.handle();
    }
    return [];
});
exports.runDeploy = (bodyFunctions, inputs) => __awaiter(void 0, void 0, void 0, function* () {
    var _c;
    const cliKey = (_c = project_utils_1.getApplicationClikeynput(inputs)) === null || _c === void 0 ? void 0 : _c.value;
    if (cliKey != '') {
        const api = new ApiService(cliKey)
            .setBody(bodyFunctions)
            .action(url_api_1.Action.DEPLOY);
        return api.handle();
    }
    new Error('check your cliKey please !!');
});
exports.runStart = (bodyFunctions, inputs) => __awaiter(void 0, void 0, void 0, function* () {
    var _d;
    const cliKey = (_d = project_utils_1.getApplicationClikeynput(inputs)) === null || _d === void 0 ? void 0 : _d.value;
    if (cliKey != '') {
        const api = new ApiService(cliKey)
            .setBody(bodyFunctions)
            .action(url_api_1.Action.START);
        return api.handle();
    }
    new Error('check your cliKey please !!');
});
exports.runStop = (bodyFunctions, inputs) => __awaiter(void 0, void 0, void 0, function* () {
    var _e;
    const cliKey = (_e = project_utils_1.getApplicationClikeynput(inputs)) === null || _e === void 0 ? void 0 : _e.value;
    if (cliKey != '') {
        const api = new ApiService(cliKey)
            .setBody(bodyFunctions)
            .action(url_api_1.Action.STOP);
        return api.handle();
    }
    new Error('check your cliKey please !!');
});
exports.runLogs = (bodyFunctions, inputs) => __awaiter(void 0, void 0, void 0, function* () {
    var _f;
    const cliKey = (_f = project_utils_1.getApplicationClikeynput(inputs)) === null || _f === void 0 ? void 0 : _f.value;
    if (cliKey != '') {
        const api = new ApiService(cliKey)
            .setBody(bodyFunctions)
            .action(url_api_1.Action.LOGS);
        return api.handle();
    }
    new Error('check your cliKey please !!');
});
exports.runStatus = (bodyFunctions, inputs) => __awaiter(void 0, void 0, void 0, function* () {
    var _g;
    const cliKey = (_g = project_utils_1.getApplicationClikeynput(inputs)) === null || _g === void 0 ? void 0 : _g.value;
    if (cliKey != '') {
        const api = new ApiService(cliKey)
            .setBody(bodyFunctions)
            .action(url_api_1.Action.STATUS);
        return api.handle();
    }
    new Error('check your cliKey please !!');
});
exports.runReplicas = (bodyFunctions, inputs) => __awaiter(void 0, void 0, void 0, function* () {
    var _h;
    const cliKey = (_h = project_utils_1.getApplicationClikeynput(inputs)) === null || _h === void 0 ? void 0 : _h.value;
    if (cliKey != '') {
        const api = new ApiService(cliKey)
            .setBody(bodyFunctions)
            .action(url_api_1.Action.REPLICAS);
        return api.handle();
    }
    new Error('check your cliKey please !!');
});
