"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.printCheckMessage = exports.isInRootDirectory = exports.getApplicationExecutedNameInput = exports.getApplicationClusterInput = exports.getApplicationNamespaceInput = exports.askForMemory = exports.selectRuntime = exports.selectReplicas = exports.selectMemory = exports.mapSchematicOptions = exports.updatePackageJson = exports.clikey = exports.loadEnv = exports.idNamespace = exports.namespace = exports.executedName = exports.functionsId = exports.getAppName = exports.functionsName = exports.runtime = exports.memory = exports.replicas = exports.loadDependencies = exports.loadSourceCode = exports.relativeSourceRoot = exports.getInputsValue = exports.getApplicationClikeynput = exports.isEmpty = exports.replaceInputMissingInformation = exports.retrieveCols = exports.print = exports.moveDefaultProjectToStart = exports.askForProjectName = exports.shouldGenerateSpec = exports.getApplicationNameInput = exports.shouldAskForProject = void 0;
const inquirer = require("inquirer");
const get_value_or_default_1 = require("../compiler/helpers/get-value-or-default");
const configuration_1 = require("../configuration");
const questions_1 = require("../questions/questions");
const schematics_1 = require("../schematics");
const chalk = require("chalk");
const path_1 = require("path");
const readers_1 = require("../readers");
const ui_1 = require("../ui");
const runtime_enums_1 = require("../runtime/runtime.enums");
const replicas_enums_1 = require("../replicas/replicas.enums");
const memory_enums_1 = require("../memory/memory.enums");
const child_process_1 = require("child_process");
const fileSystemReader = new readers_1.FileSystemReader(process.cwd());
const loader = new configuration_1.MicrofunctionConfigurationLoader(fileSystemReader);
const entryFilename = `package.${'json'}`;
function shouldAskForProject(schematic, configurationProjects, appName) {
    return (['app', 'sub-app', 'library', 'lib'].includes(schematic) === false &&
        configurationProjects &&
        Object.entries(configurationProjects).length !== 0 &&
        !appName);
}
exports.shouldAskForProject = shouldAskForProject;
exports.getApplicationNameInput = (inputs) => inputs.find(input => input.name === 'name');
function shouldGenerateSpec(configuration, schematic, appName, specValue, specPassedAsInput) {
    if (specPassedAsInput === true || specPassedAsInput === undefined) {
        // CLI parameters has the highest priority
        return specValue;
    }
    let specConfiguration = get_value_or_default_1.getValueOrDefault(configuration, 'generateOptions.spec', appName || '');
    if (typeof specConfiguration === 'boolean') {
        return specConfiguration;
    }
    if (typeof specConfiguration === 'object' &&
        specConfiguration[schematic] !== undefined) {
        return specConfiguration[schematic];
    }
    if (typeof specConfiguration === 'object' && appName) {
        // The appName has a generateOption spec, but not for the schematic trying to generate
        // Check if the global generateOptions has a spec to use instead
        specConfiguration = get_value_or_default_1.getValueOrDefault(configuration, 'generateOptions.spec', '');
        if (typeof specConfiguration === 'boolean') {
            return specConfiguration;
        }
        if (typeof specConfiguration === 'object' &&
            specConfiguration[schematic] !== undefined) {
            return specConfiguration[schematic];
        }
    }
    return specValue;
}
exports.shouldGenerateSpec = shouldGenerateSpec;
function askForProjectName(promptQuestion, projects) {
    return __awaiter(this, void 0, void 0, function* () {
        const questions = [
            questions_1.generateSelect('appName')(promptQuestion)(projects),
        ];
        const prompt = inquirer.createPromptModule();
        return prompt(questions);
    });
}
exports.askForProjectName = askForProjectName;
function moveDefaultProjectToStart(configuration, defaultProjectName, defaultLabel) {
    let projects = Object.keys(configuration.projects);
    if (configuration.sourceRoot !== 'src') {
        projects = projects.filter(p => p !== defaultProjectName.replace(defaultLabel, ''));
    }
    projects.unshift(defaultProjectName);
    return projects;
}
exports.moveDefaultProjectToStart = moveDefaultProjectToStart;
exports.print = (color = null) => (str = '') => {
    const terminalCols = exports.retrieveCols();
    const strLength = str.replace(/\u001b\[[0-9]{2}m/g, '').length;
    const leftPaddingLength = Math.floor((terminalCols - strLength) / 2);
    const leftPadding = ' '.repeat(Math.max(leftPaddingLength, 0));
    if (color) {
        str = chalk[color](str);
    }
    console.log(leftPadding, str);
};
exports.retrieveCols = () => {
    const defaultCols = 80;
    try {
        const terminalCols = child_process_1.execSync('tput cols', {
            stdio: ['pipe', 'pipe', 'ignore'],
        });
        return parseInt(terminalCols.toString(), 10) || defaultCols;
    }
    catch (_a) {
        return defaultCols;
    }
};
exports.replaceInputMissingInformation = (inputs, answers) => {
    return inputs.map(input => (input.value =
        input.value !== undefined && input.value !== '' ? input.value : answers[input.name] || ''));
};
exports.isEmpty = (value) => {
    return value === "" || value === null || value === undefined;
};
exports.getApplicationClikeynput = (inputs) => inputs.find(input => input.name === 'cliKey');
exports.getInputsValue = (inputs, name) => { var _a; return (_a = inputs.find(input => input.name === name)) === null || _a === void 0 ? void 0 : _a.value; };
exports.relativeSourceRoot = (inputs) => exports.getInputsValue(inputs, 'name') || '';
exports.loadSourceCode = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    const sourceRoot = '/src';
    const entryFilename = `index.${'js'}`;
    answers['sourceCode'] = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs).concat(sourceRoot), `${entryFilename}`), configuration_1.TypeFile.TEXT);
    exports.replaceInputMissingInformation(inputs, answers);
});
exports.loadDependencies = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
    const dependencies = packageJson.dependencies;
    answers['packageJson'] = JSON.stringify({ dependencies });
    exports.replaceInputMissingInformation(inputs, answers);
});
exports.replicas = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    let replicas = exports.getInputsValue(inputs, 'replicas');
    if (replicas === undefined || replicas == '') {
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        replicas = packageJson.replicas;
        answers['replicas'] = replicas;
        exports.replaceInputMissingInformation(inputs, answers);
    }
    if (replicas === undefined || replicas == '') {
        yield exports.selectReplicas(inputs);
    }
});
exports.memory = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    let memory = exports.getInputsValue(inputs, 'memory');
    if (memory === undefined || memory == '') {
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        memory = packageJson.memory;
        answers['memory'] = memory;
        exports.replaceInputMissingInformation(inputs, answers);
    }
    if (memory === undefined || memory == '') {
        yield exports.selectMemory(inputs);
    }
});
exports.runtime = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    let runtime = exports.getInputsValue(inputs, 'runtime');
    if (runtime === undefined || runtime == '') {
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        runtime = packageJson.runtime;
        answers['runtime'] = runtime;
        exports.replaceInputMissingInformation(inputs, answers);
    }
    if (runtime === undefined || runtime == '') {
        yield exports.selectRuntime(inputs);
    }
});
exports.functionsName = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    const functions = exports.getInputsValue(inputs, 'functions');
    if (functions === undefined || functions == '') {
        const entryFilename = `package.${'json'}`;
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        const name = packageJson.name;
        answers['functions'] = name.toLowerCase();
        exports.replaceInputMissingInformation(inputs, answers);
    }
});
exports.getAppName = (inputs) => __awaiter(void 0, void 0, void 0, function* () {
    let name = exports.getInputsValue(inputs, 'name');
    if (name === undefined || name == '') {
        const entryFilename = `package.${'json'}`;
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        name = packageJson.name;
    }
    return name;
});
exports.functionsId = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    let id = exports.getInputsValue(inputs, 'id');
    if (id === undefined || id == '') {
        const entryFilename = `package.${'json'}`;
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        answers['id'] = id = packageJson.id;
        exports.replaceInputMissingInformation(inputs, answers);
    }
    return id;
});
exports.executedName = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    let executedName = exports.getInputsValue(inputs, 'executedName');
    if (executedName === undefined || executedName == '') {
        const entryFilename = `package.${'json'}`;
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        answers['executedName'] = executedName = packageJson.executedName;
        exports.replaceInputMissingInformation(inputs, answers);
    }
    return executedName;
});
exports.namespace = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    let namespace = exports.getInputsValue(inputs, 'namespace');
    if (namespace === undefined || namespace == '') {
        const entryFilename = `package.${'json'}`;
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        answers['namespace'] = namespace = (_a = packageJson.namespace) === null || _a === void 0 ? void 0 : _a.name;
        exports.replaceInputMissingInformation(inputs, answers);
    }
    return namespace;
});
exports.idNamespace = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    let idNamespace = exports.getInputsValue(inputs, 'idNamespace');
    if (idNamespace === undefined || idNamespace == '') {
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        answers['idNamespace'] = idNamespace = (_b = packageJson.namespace) === null || _b === void 0 ? void 0 : _b.id;
        exports.replaceInputMissingInformation(inputs, answers);
    }
    return idNamespace;
});
exports.loadEnv = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    const entryFilename = `.env`;
    const envsource = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`), configuration_1.TypeFile.TEXT);
    answers['environments'] = parsesObject(envsource);
    exports.replaceInputMissingInformation(inputs, answers);
});
exports.clikey = (inputs, answers) => __awaiter(void 0, void 0, void 0, function* () {
    let clikey = exports.getInputsValue(inputs, 'cliKey');
    if (clikey === undefined || clikey == '') {
        const packageJson = yield loader.load(path_1.join(exports.relativeSourceRoot(inputs), `${entryFilename}`));
        answers['cliKey'] = clikey = packageJson.cliKey;
        exports.replaceInputMissingInformation(inputs, answers);
    }
    return clikey;
});
exports.updatePackageJson = (args, options) => __awaiter(void 0, void 0, void 0, function* () {
    var _c;
    const collectionName = (_c = options.find(option => option.name === 'collection' && option.value != null)) === null || _c === void 0 ? void 0 : _c.value;
    const collection = schematics_1.CollectionFactory.create(collectionName || schematics_1.Collection.MICROFUNCTION);
    const schematicOptions = exports.mapSchematicOptions(args.concat(options));
    yield collection.execute('sub-app', schematicOptions);
});
exports.mapSchematicOptions = (inputs) => {
    const excludedInputNames = ['schematic', 'spec'];
    const options = [];
    inputs.forEach(input => {
        if (!excludedInputNames.includes(input.name) && input.value !== undefined) {
            options.push(new schematics_1.SchematicOption(input.name, input.value));
        }
    });
    return options;
};
exports.selectMemory = (inputs) => __awaiter(void 0, void 0, void 0, function* () {
    const answers = yield exports.askForMemory();
    exports.replaceInputMissingInformation(inputs, answers);
});
exports.selectReplicas = (inputs) => __awaiter(void 0, void 0, void 0, function* () {
    const answers = yield askForReplicas();
    exports.replaceInputMissingInformation(inputs, answers);
});
exports.selectRuntime = (inputs) => __awaiter(void 0, void 0, void 0, function* () {
    const answers = yield askForRuntime();
    exports.replaceInputMissingInformation(inputs, answers);
});
exports.askForMemory = () => __awaiter(void 0, void 0, void 0, function* () {
    const questions = [
        questions_1.generateSelect('memory')(ui_1.MESSAGES.MEMORY_QUESTION)([
            memory_enums_1.MemoryEnums.M32,
            memory_enums_1.MemoryEnums.M64,
            memory_enums_1.MemoryEnums.M128,
            memory_enums_1.MemoryEnums.M256,
            memory_enums_1.MemoryEnums.M512,
        ]),
    ];
    const prompt = inquirer.createPromptModule();
    return yield prompt(questions);
});
const askForReplicas = () => __awaiter(void 0, void 0, void 0, function* () {
    const questions = [
        questions_1.generateSelect('replicas')(ui_1.MESSAGES.REPLICAS_QUESTION)([
            replicas_enums_1.ReplicasEnums.R_1,
            replicas_enums_1.ReplicasEnums.R_2,
            replicas_enums_1.ReplicasEnums.R_3,
            replicas_enums_1.ReplicasEnums.R_4,
        ]),
    ];
    const prompt = inquirer.createPromptModule();
    return yield prompt(questions);
});
const askForRuntime = () => __awaiter(void 0, void 0, void 0, function* () {
    const questions = [
        questions_1.generateSelect('runtime')(ui_1.MESSAGES.RUNTIME_QUESTION)([
            runtime_enums_1.RuntimeEnums.NODE_12,
            runtime_enums_1.RuntimeEnums.NODE_10,
            runtime_enums_1.RuntimeEnums.NODE_8,
            runtime_enums_1.RuntimeEnums.NODE_6,
        ]),
    ];
    const prompt = inquirer.createPromptModule();
    return yield prompt(questions);
});
exports.getApplicationNamespaceInput = (inputs) => inputs.find(input => input.name === 'namespace');
exports.getApplicationClusterInput = (inputs) => inputs.find(input => input.name === 'cluster');
exports.getApplicationExecutedNameInput = (inputs) => inputs.find(input => input.name === 'executedName');
exports.isInRootDirectory = (inputs, extraFiles) => __awaiter(void 0, void 0, void 0, function* () {
    const files = ['microfunctions-cli.json'].concat(extraFiles || []);
    try {
        const file = yield loader.load(`microfunctions-cli.json`);
        return file;
    }
    catch (e) {
        throw new Error(' Please, ensure that you are running this command in the appropriate directory (inside microfunctions workspace).');
    }
});
// Parses src into an Object
const parsesObject = (src) => {
    const NEWLINE = '\n';
    const RE_INI_KEY_VAL = /^\s*([\w.-]+)\s*=\s*(.*)?\s*$/;
    const RE_NEWLINES = /\\n/g;
    const NEWLINES_MATCH = /\n|\r|\r\n/;
    const obj = [];
    // convert Buffers before splitting into lines and processing
    src.toString().split(NEWLINES_MATCH).forEach(function (line) {
        // matching "KEY' and 'VAL' in 'KEY=VAL'
        const keyValueArr = line.match(RE_INI_KEY_VAL);
        // matched?
        if (keyValueArr != null) {
            const name = keyValueArr[1];
            // default undefined or missing values to empty string
            let value = (keyValueArr[2] || '');
            const end = value.length - 1;
            const isDoubleQuoted = value[0] === '"' && value[end] === '"';
            const isSingleQuoted = value[0] === '\'' && value[end] === '\'';
            // if single or double quoted, remove quotes
            if (isSingleQuoted || isDoubleQuoted) {
                value = value.substring(1, end);
                // if double quoted, expand newlines
                if (isDoubleQuoted) {
                    value = value.replace(RE_NEWLINES, NEWLINE);
                }
            }
            else {
                // remove surrounding whitespace
                value = value.trim();
            }
            obj.push({ name, value });
        }
    });
    return obj;
};
exports.printCheckMessage = (message) => {
    const emptyLine = exports.print();
    const red = exports.print('red');
    red(message);
    exports.print()(`${chalk.bold(`${ui_1.EMOJIS.WINE}  blog:`)} ${chalk.underline('https://microfunctions.io/blog')}`);
    emptyLine();
};
